      Program ba_gm_tmr

* Program to compute ground motions from
* BA provisional GMPE

* Uses control file:
*! Control file for program ba_gm_tmr.for
*!Header to add to output file (no "!" at beginning!)
* [blank]
*!file containing regression coefficients:
*  ba_02apr07_usnr.out
*!name of output file:
*  ba_gm_tmr_m6.69_7.80_v30_760_mech_r.out
*!   T     M     R v30 imech (imech=-1,0,1,2 for u,s,n,r)
* -1.0  6.69   0.1 760     2  
* -1.0  6.69 400.0 760     2  
*  0.0  6.69   0.1 760     2  
*  0.0  6.69 400.0 760     2  
*  0.1  6.69   0.1 760     2  
*  0.1  6.69 400.0 760     2  
*  0.2  6.69   0.1 760     2  
*  0.2  6.69 400.0 760     2  
*  1.0  6.69   0.1 760     2  
*  1.0  6.69 400.0 760     2  
*  3.0  6.69   0.1 760     2  
*  3.0  6.69 400.0 760     2  
* -1.0  7.80   0.1 760     2  
* -1.0  7.80 400.0 760     2  
*  0.0  7.80   0.1 760     2  
*  0.0  7.80 400.0 760     2  
*  0.1  7.80   0.1 760     2  
*  0.1  7.80 400.0 760     2  
*  0.2  7.80   0.1 760     2  
*  0.2  7.80 400.0 760     2  
*  1.0  7.80   0.1 760     2  
*  1.0  7.80 400.0 760     2  
*  3.0  7.80   0.1 760     2  
*  3.0  7.80 400.0 760     2  
* stop
    
* Dates: 03/03/06 - written by D.M. Boore, based on ba_gm_vs_r
*        07/22/06 - Allow for cubic smoothing of nonlinear amps.
*        10/10/06 - Allow for more periods.
*        11/12/06 - Use buf4 instead of buf(1:4)
*        12/01/06 - Changed column headings to be more accurate in
*                   terms of the standard deviation, and added a column
*                   of e^sig.
*        02/03/07 - Allow for more periods.
*        02/03/07 - Delete path input (not needed)
*        03/15/07 - Print fm, fd, fs
*        03/26/07 - Read over header lines (preceded by "!") in the 
*                   regression coefficient file.
*        10/30/07 - Skip calculation for nomral fault at T=10 s if
*                   e3 = 999.99. 

    
      real per    
      
      character buf*80, buf4*4, f_ctl*80, f_file*80, 
     :          f_coeff*120, f_out*200,
     :          header4output*80, mech*1,
     :          format_out*80
     
      
      character coeff_head(10)*3
      
      character cmnts2skip(200)*79
      
      logical file_exist 
      
      real per_stage1(120), per_stage2(120)
      
      real per_gmpe(120), e_gmpe(8,120), amh_gmpe(120), 
     :     c_gmpe(4,120), amref_gmpe(120), rref_gmpe(120), h_gmpe(120), 
     :     blin(120), vref(120), b1(120), b2(120), v1(120), v2(120),
     :     a1(120), pga_low(120), a2(120),
     :     sig1_gmpe(120), 
     :     sig2_gmpe_u(120), sigt_gmpe_u(120),
     :     sig2_gmpe_m(120), sigt_gmpe_m(120),
     :     rdum(120)
        
      real e_pga4nl(8), amh_pga4nl, 
     :     c_pga4nl(4), amref_pga4nl, rref_pga4nl, h_pga4nl, 
     :     sigt_pga4nl
        
      real m, logy_pred, mstart, mstop, dmag, rjb
      real y, sigy 
      
      integer nmag
 
      pi = 4.0*atan(1.0)
      twopi = 2.0 * pi
      
      ncoeff_max = 5
      nper_max = 120
      
      ncoeff_stage1 = 4      
      ncoeff_stage2 = 8

*DEBUG
      call get_lun(nu_debug)
      open(unit=nu_debug,file='debug.out',status='unknown')
*DEBUG

      do j = 1, nper_max
        sig1_gmpe(j) = 0.0
        sig2_gmpe_u(j) = 0.0
        sigt_gmpe_u(j) = 0.0
        sig2_gmpe_m(j) = 0.0
        sigt_gmpe_m(j) = 0.0
      end do

      do i = 1, ncoeff_stage1
        c_pga4nl(i) = 0.0
        do j = 1, nper_max
          c_gmpe(i,j) = 0.0
        end do
      end do
      
      do i = 1, ncoeff_stage2
        e_pga4nl(i) = 0.0
        do j = 1, nper_max
          e_gmpe(i,j) = 0.0
        end do
      end do
      
      file_exist = .false.
      do while (.not. file_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = ba_gm_tmr.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'ba_gm_tmr.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=file_exist)
        if (.not. file_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      
      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)  
      header4output = ' '
      read(nu_ctl,'(a)') header4output
      call trim_c(header4output,nc_h4out)
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      buf = ' '
      read(nu_ctl,'(a)') buf
      call trim_c(buf,nc_buf)
      f_coeff  = ' '
      f_coeff  = buf(1:nc_buf)
      call trim_c(f_coeff, nc_f_coeff)
 
      
* Open coefficient file:

      print *,' f_coeff  = '//
     :          f_coeff(1:nc_f_coeff)
      
      call get_lun(nu_coeff)
      open(unit=nu_coeff,file=f_coeff(1:nc_f_coeff),
     :     status='unknown')
     
      call skipcmnt(nu_coeff,cmnts2skip,nc_cmnts2skip)      
      call skip(nu_coeff,1)  ! The column header row is NOT preceded with '!'
                             ! because might want to open the coefficient file
                             ! in Excel.
           
* read coefficients for pga4nl:

      read(nu_coeff,*) 
     :   per_pga4nl, (e_pga4nl(j), j = 1, ncoeff_stage2),
     :   amh_pga4nl, (c_pga4nl(j), j = 1, ncoeff_stage1), 
     :   amref_pga4nl, rref_pga4nl, h_pga4nl 

 
* read coefficients for regression coefficients:

      i = 0
3001  continue

      i = i + 1
      
      read(nu_coeff,*,end=3999) 
     :   per_gmpe(i), (e_gmpe(j,i), j = 1, ncoeff_stage2),
     :   amh_gmpe(i), (c_gmpe(j,i), j = 1, ncoeff_stage1), 
     :   amref_gmpe(i), rref_gmpe(i), h_gmpe(i), 
     :   blin(i), vref(i), b1(i), b2(i), v1(i), v2(i), 
     :   a1(i), pga_low(i), a2(i),
     :   sig1_gmpe(i), 
     :   sig2_gmpe_u(i), sigt_gmpe_u(i),
     :   sig2_gmpe_m(i), sigt_gmpe_m(i)
      
       
      goto 3001
3999  continue
      
      close(nu_coeff)
      
      nper_gmpe = i-1
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      f_out = ' '
      read(nu_ctl,'(a)') f_out
      call trim_c(f_out,nc_f_out)
      call get_lun(nu_out)
      open(unit=nu_out,file=f_out(1:nc_f_out),status='unknown')

      write(nu_out,'(
     :                6x,a, 4x,a, 4x,a, 4x,a, 1x,a,
     :                7x,a, 5x,a, 1x,a, 1x,a, 
     :                2x,a,
     :                1x,a, 1x,a, 1x,a
     :                                  )')
     :           'T', 'M', 'Rjb', 'V30', 'imech',
     :           'Y(g)', 'Y(cgs)', 'Y/e^sig_LnY', 'Y*e^sig_LnY',
     :           'e^sig_lnY',
     :           '981*exp(fm)', '981*exp(fd)', '981*exp(fs)'

      
* Loop over values of TMR, etc:

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      

888   continue

        buf = ' '
        read(nu_ctl,'(a)') buf
        call trim_c(buf,nc_buf)
        buf4 = ' '
        buf4 = buf(1:4)
        call trim_c(buf4,nc_buf4)
        call upstr(buf4)
      
        if (buf4(1:4) .eq. 'STOP') then
          close(nu_ctl)
          close(nu_out)
          close(nu_debug)
          stop
        end if
      
        read(buf(1:nc_buf),*) per, m, rjb, v30, imech
      
        if (imech .eq. -1) then
          mech = 'U'
        else if (imech .eq. 0) then
          mech = 'S'
        else if (imech .eq. 1) then
          mech = 'N'
        else if (imech .eq. 2) then
          mech = 'R'
        else
          print *,' ERROR, imech = ',imech,
     :          ' not valid; go to next record'
          go to 888
        end if
 
        do i = 1, nper_gmpe
          if(per .eq. per_gmpe(i)) then
            index_per = i
            go to 3111
          end if
        end do
      
        print *,' per not found in gmpe coefficient file,'//
     :        ' go to next record'
        print *,' per = ', per
        
        go to 888
3111    continue  

* Check for normal fault, T = 10:
        if (mech(1:1) .eq. 'N' .and. per .eq. 10.0) then
          if (e_gmpe(3,index_per) .eq. 999.99) then
            print *, ' Normal fault motion for'//
     :               ' T = 10 s is undefined, skipping'//
     :               ' to next set of predictor variables'
            go to 888
          end if
        end if
        
 
        if (mech(1:1) .eq. 'U') then
          sigt = sigt_gmpe_u(index_per)
        else
          sigt = sigt_gmpe_m(index_per)
        end if

        call ba_gm_sub4y(
     :        per, m, rjb, r,
     :        mech(1:1),v30,y,expsiglny,
     :        e_gmpe(1,index_per), amh_gmpe(index_per), 
     :        c_gmpe(1,index_per), amref_gmpe(index_per),
     :        rref_gmpe(index_per), h_gmpe(index_per), 
     :        vref(index_per), 
     :        a1(index_per), pga_low(index_per), a2(index_per),
     :        sigt,
     :        e_pga4nl, amh_pga4nl, 
     :        c_pga4nl, amref_pga4nl,
     :        rref_pga4nl, h_pga4nl,
     :        pga4nl, fm_pga4nl, fd_pga4nl, fs_pga4nl,
     :        bnl, amp_lin, amp_nl, amp_total,
     :        fm_gmpe, fd_gmpe, fs_gmpe)

 
* Convert from g to cgs units:

        if (per .ge. 0.0) then
          yg   = y
          ycgs = 981.0 * yg
        else
          yg   = y 
          ycgs = y
        end if

        write(nu_out,'(
     :      1x,f6.3, 1x,f4.2, 1x,f6.2, 1x,f6.1, 4x,i2,
     :      1p,2(1x,e10.3), 2(2x,e10.3),
     :           1x,e10.3,
     :         3(2x,e10.3))')
     :      per, m, rjb, v30, imech, 
     :      yg, ycgs, ycgs/expsiglny, ycgs*expsiglny,
     :      expsiglny,
     :      981.0*exp(fm_gmpe), 981.0*exp(fd_gmpe), 981.0*exp(fs_gmpe)
     
      go to 888 ! loop back for another record

      end

      include 'ba_gm_sub4y.for'
      
      include '\forprogs\skipcmnt.for'
        
      include '\forprogs\get_lun.for'
      include '\forprogs\lin_interp.for'
      include '\forprogs\locate.for'
      include '\forprogs\skip.for'
      include '\forprogs\upstr.for'
      include '\forprogs\trim_c.for'

 
